
//////////////////////////////////////////////////////////////////////////////////////////////////
//  cameras.wdl 1st and 3rd person cameras
//////////////////////////////////////////////////////////////////////////////////////////////////
// variables
////////////////////////////////////////////////////////////////////
var cam_mode = 0; // 0 meaning 1st Person, 1 meaning 3rd Person
var eye_height = 20;
var tilt_1st = 0;
var cam_turnspeed = .5;
var max_tilt_1st = 40;
var dist_planar = 300; // distance from the player
var cam_angle = 0;
var dist_total = 300; // Change THIS value to zoom in or out
var tilt_3rd = 0;
var dist_traced;
var camera_loc[3] = 155,15,0; // camera location and angles
/////////////////////////////////////////////////////////////
// views  defining views and updating cameras
/////////////////////////////////////////////////////////////
view 1st_person
{
layer = 1;
pos_x = 0;
pos_y = 0;
}
//////////////////
view 3rd_person
{
layer = 1;
pos_x = 0;
pos_y = 0;
}
///////////////////
function toggle_cams()
{
	if (cam_mode == 0) // Change to 3rd Person
	{ 
		1st_person.visible = off;
		3rd_person.visible = on;
		cam_mode = 1;
	}
	else // Change to 1st Person
	{ 
		3rd_person.visible = off;
		1st_person.visible = on;
		cam_mode = 0;
	}
}
///////////////////////
function validate_view()
{
	my = player;
	trace_mode = ignore_me + ignore_passable;
	dist_traced = trace (player.x, 3rd_person.x);
	if (dist_traced == 0) { return; } // No obstacles hit... fine
	if (dist_traced < dist_total)
	{
		dist_traced -= 5; // Move it out of the wall
		dist_planar = cos (tilt_3rd) * dist_traced;
		3rd_person.x = player.x - cos (cam_angle + player.pan) * dist_planar;
		3rd_person.y = player.y - sin (cam_angle + player.pan) * dist_planar;
		3rd_person.z = player.z + sin (tilt_3rd) * dist_traced;
	}
}
//////////////////////
function update_views()
{
	if (cam_mode == 0)
	{
		1st_person.x = player.x;
		1st_person.y = player.y;
		1st_person.z = player.z + eye_height;
		1st_person.pan = player.pan;
		1st_person.roll = player.roll;
		1st_person.tilt = player.tilt + tilt_1st;
	}
	else
	{
		dist_planar = cos (tilt_3rd) * dist_total;
		3rd_person.x = player.x - cos (cam_angle + player.pan) * dist_planar;
		3rd_person.y = player.y - sin (cam_angle + player.pan) * dist_planar;
		3rd_person.z = player.z + sin (tilt_3rd) * dist_total;
		3rd_person.pan = cam_angle + player.pan;
		3rd_person.roll = 0;
		3rd_person.tilt = - tilt_3rd;
		validate_view();
		
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// adjust 1st person view tilt
//////////////////////////////////////////////////////////////////////////////////////////////////
function handle_pageup()
{
	while (key_pgup)
	{
		if (tilt_1st < max_tilt_1st) { tilt_1st += cam_turnspeed; }
		wait(1);
	}
}
/////////////////////
function handle_pagedown()
{
	while (key_pgdn)
	{
		if (tilt_1st > -max_tilt_1st) { tilt_1st -= cam_turnspeed; }
		wait(1);
	}
}
function level_tilt()
{
	tilt_1st = 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//  adjust 3rd person view
//////////////////////////////////////////////////////////////////////////////////////////////////
function cam3rd_angle()
{
	while (key_a)
	{
		cam_angle -= cam_turnspeed;
		if (cam_angle < 0 ) {cam_angle = 359;}
		if (cam_angle > 359) {cam_angle = 0;}
		wait(1);
	}
	while (key_d)
	{
		cam_angle += cam_turnspeed;
		if (cam_angle < 0 ) {cam_angle = 359;}
		if (cam_angle > 359) {cam_angle = 0;}
		wait(1);
	}
}
///////////////////////
function cam3d_tilt()
{
	while (key_w)
	{
		if (tilt_3rd < 65 ) { tilt_3rd += cam_turnspeed;}
		wait(1);
	}
	while (key_x)
	{
		if (tilt_3rd > 0 ) { tilt_3rd -= cam_turnspeed;}
		wait(1);
	}
}
////////////////////////
function cam3d_dist()
{
	while (key_q)
	{
		if (dist_total > 40  ) { dist_total -= cam_turnspeed;}
		wait(1);
	}
	while (key_z)
	{
		if (dist_total < 300 ) { dist_total += cam_turnspeed;}
		wait(1);
	}
}
///////////////////////
function cam3d_center()
{
	cam_angle = 0;
}

function default3d()
{
	dist_total = camera_loc[0];
	tilt_3rd = camera_loc[1];
	cam_angle = camera_loc[2];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// call this function in your characters action loop
/////////////////////////////////////////////////////////////////////////////////////////////////
function 1st_3rdcameras ()
{
	on_pgup = handle_pageup;
	on_pgdn = handle_pagedown;
	on_home = level_tilt;
	on_f9 = toggle_cams;
	on_f10 = default3d;
	on_a = cam3rd_angle;
	on_d = cam3rd_angle;
	on_w = cam3d_tilt;
	on_x = cam3d_tilt;
	on_q = cam3d_dist;
	on_z = cam3d_dist;
	on_s = cam3d_center;
	if (mouse_pos.x < 5) {cam_angle -= cam_turnspeed;}
	if (mouse_pos.x > 794) { cam_angle += cam_turnspeed;}
	if (mouse_pos.y <5) { if (tilt_3rd < 65 ) { tilt_3rd += cam_turnspeed;}	}
	if (mouse_pos.y > 594) { if (tilt_3rd > 0 ) { tilt_3rd -= cam_turnspeed;}	}
	if (sign(mickey.z) == 1) { if (dist_total > 40  ) { dist_total -= cam_turnspeed*8;}	}
	if (sign(mickey.z) == -1) { if (dist_total < 300 ) { dist_total += cam_turnspeed*8;}	}
	update_views();
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// call this functio in your main() to set up camera views
//////////////////////////////////////////////////////////////////////////////////////////////////
function init_cameras()
{
	camera.visible = off;
	1st_person.size_x = screen_size.x;
	1st_person.size_y = screen_size.y;
	1st_person.genius = player;
	1st_person.visible = on;
	3rd_person.size_x = screen_size.x;
	3rd_person.size_y = screen_size.y;
	3rd_person.genius = player;
	3rd_person.visible = off;
}
//////////////////////////////////////////////////////////////////////////////////////////////////

